/*Environment generation code v2.11., author: Ryan Kirwan.*/

/*GENERATING OBSTACLE SET FOR EXPLICIT ENVIRONMENT
***************************************************
This C code is used to generate obstacles for our Explicit model environments. 
Environments are circular areas, mapped by polar coordinates measured from 
their centres. We define an environmental complexity (default of 155units), 
which is the minimum spacing between obstacles. The distance to the edge of 
an environment that an obstacle can be placed is restricted because an agent 
can wrap around the environment's edges, and an agent should not wrap around 
the environment appearing in/on an obstacle. To account for this issue we define 
the minimum spacing from the side of the environment as ENVI_COMPLEXITY/2. 
(E.g., with envi rad of 200 and envi complex of 155 an ob cannot be farther 
from the envi centre than 122units -- minus 78units from edge.) 
Additionally, obstacles cannot be placed within 90units an envi centre 
-- based on initial agent placement restrictions, having no instant ob contact.
Obstacles are attempted to be placed either clockwise/anti from the previously 
placed ob (this decided randomly).*/

/* HOW TO USE
**************
First compile with math lib e.g., "gcc explObGenV000.c -lm".
Then run the a.out file ("./a.out").
The results will be preduced in the text file "validEnviObCoords.txt".

Parameters (adjustable):
	"NUMOFENVIS" : number of environments generated.

	"ENVIRADIUS" : radius of the environments.

	"MAXOBNUM"   : maximum number of obs to be place in an envi 
	(we attempt to place as many obs as possilbe, up to the max).

	"MINDIST"    : minimum distance between obs
	(no two obstacles can be closer than this dist to one another).
**************/

#include <math.h>
#include <stdio.h> 
#include <time.h>
#include <stdlib.h>
#include "obGenFunctions.h"

int main() {

	FILE *fp;
	fp = fopen("validEnviObCoords.txt", "w");

/**SET NUMBER OF ENVIRONMENTS**/
int NUMOFENVIS = 3;
/******************************/

/**SET ENVIRONMENT SIZE**/
int ENVIRADIUS = 200;
/************************/

/**SET MAXIMUM NUMBER OF OBSTACLES**/
int MAXOBNUM= 10;
/***********************************/

/**SET MINIMUM DISTANCE BETWEEN OBSTACLES**/
int MINDIST = 155;
/******************************************/
	
	/*Calc min dist from edge that ob can be 
	placed based on envi size and min dist between obs.*/
	int MINFROMEDGE = ENVIRADIUS-((MINDIST/2)+1);
	/*Random gen seed for init ob coord and 
	placement preference from obs (clock/anti).*/
	srand(time(NULL));
	int antiClock = rand() % 2;
	/***************************************************/
	
	int envi = 0; 
	int i = 0;
	int j = 0;
	int k = 0;
	int resetCount = 0;
	int toClose = 0;
	int fail = 0;	
	int test = 0;

	int enviDist = 0;
	int enviAng = 0;
	int fromObAng = 10; /*Angle ob is to previous one*/
	int angFrom = 0;
	int obCount = 0;
	int debug = 0;

	coord_t nullifier; 
	nullifier.d = 0;
	nullifier.a = 0;
	coord_t prevOb;

	fprintf(fp, "\nEnvironment generation code v2.11., author: Ryan Kirwan.\n\n");

	fprintf(fp, "Environmental parameters: \n");
	fprintf(fp, "	Environment radius: %i.\n", ENVIRADIUS);
	fprintf(fp, "	Maximum number of obstacles requested: %i.\n", MAXOBNUM);
	fprintf(fp, "	Minimum distance between obstacles: %i.\n\n", MINDIST);
	
	for (envi=1; envi<=NUMOFENVIS; envi++) {
		
		/*Reset vars for new Envi.*/
		i = 0; j = 0; k = 0; toClose = 0; fail = 0; obCount = 0;
		enviDist = rand() % MINFROMEDGE;
		/*There shall be no obs within 90 of the centre*/
		if (enviDist <= 90) {enviDist = enviDist+90;}
		enviAng = rand() % 360; 
		prevOb = setCoord(enviDist, enviAng);
		arrObs[0] = setCoord(enviDist, enviAng);

		fprintf(fp, "Enviroment E%i obstacles: \n", envi);
		fprintf(fp, "	arrObs[%d].d = %d; arrObs[%d].a = %d; \n", i, arrObs[0].d, i, arrObs[0].a);

		for (i=1; i<=MAXOBNUM; i++) {

			for (j=0; j<360; j++) {
				toClose = 1;
	
				if (antiClock==1) {angFrom = (359 - j);}
				else {angFrom = j;}

				arrObs[i] = genOb(prevOb, angFrom);

				if (arrObs[i].d <= MINFROMEDGE) {
					toClose = 0;

					/*Loop through all preveous obstacles to check distance.*/
					for (k=0; k<(i-1); k++) {

						test = testDist(arrObs[i], arrObs[k]);	
	
						if (test < MINDIST) { 
							toClose = 1; 
							break;
						}
					}
				}
				if (toClose==0) { 

					antiClock = rand() % 2;
					break;
				}
				if ((toClose==1) && (j==359)) {fail = 1;}
			}

			if (fail==1) { fprintf(fp, "\n	Cannot place more obstacles for this environment.\n\n"); break;}
			else {obCount++;}
	
			if (obCount >= MAXOBNUM) { fprintf(fp, "\n	Successfully generated %i obstacles.\n\n", obCount); break;}
		
			/*arrObs[0].d=80; arrObs[0].a=90;*/
			fprintf(fp, "	arrObs[%d].d = %d; arrObs[%d].a = %d; \n", i, arrObs[i].d, i, arrObs[i].a);
			/*Reset to the new coords of the latest obstacle.*/
			fromObAng = fromObAng + 90;
			/*Save the previous Object coords for the obGen function.*/
			prevOb = setCoord(arrObs[i].d, arrObs[i].a);
		}
	
	}
	
	fclose(fp);
	return 0;
}
